==> sigutils-2.4.6.tar.gz: sigutils-2.4.6/SigUtils/registro.py <==
# -*- encoding: utf-8 -*-
#- coding: utf-8
__Autor__ = "Tiago Prates"
__Empresa__ = "Sig Informatica Ltda"
__Site__ = "http://www.siginformatica.com.br"
__Versao__ = "1.00"
__DataAlteracao__ = "17/07/2013"

import sys
import json
from glob import iglob
from ordereddict import OrderedDict
import os
from decimal import Decimal, InvalidOperation

if __name__ == '__main__':
    try:
        if sys.argv[1] == '-9':
            print('registro.py\t\t\tVersao:' + __Versao__ + '\tData:'+ __DataAlteracao__) + '\tCompoe e decompoe registros de arquivo texto'
        else:
            print('\n\nregistro.py - Compoe e decompoe registros de arquivo texto \n'
                  +'Versao: '+ __Versao__ + '\n'
                  +'Data: '+ __DataAlteracao__ + '\n')
    except:
        print('\n\nregistro.py - Compoe e decompoe registros de arquivo texto \n'
              +'Versao: '+ __Versao__ + '\n'
              +'Data: '+ __DataAlteracao__ + '\n')

class CampoBase(object):
    def __init__(self):
        self._valor = None

    @property
    def valor(self):
        return self._valor

    @valor.setter
    def valor(self, valor):
        self._valor = valor


    def __unicode__(self):
        if self.valor is None:
            if self.default is not None:
                if self.decimais:
                    self.valor = Decimal('{0:0.{1}f}'.format(self.default,
                        self.decimais))
                else:
                    self.valor = self.default
                #            else:
                #                raise errors.CampoObrigatorioError(self.nome)

        if self.formato == 'alfa' or self.decimais:
            if self.decimais:
                valor = unicode(self.valor).replace('.', '')
                chars_faltantes = self.digitos - len(valor)
                return (u'0' * chars_faltantes) + valor
            else:
                chars_faltantes = self.digitos - len(self.valor)
                return self.valor + (u' ' * chars_faltantes)

        return u'{0:0{1}d}'.format(self.valor, self.digitos)

    def __repr__(self):
        return unicode(self)

    def __set__(self, instance, value):
        self.valor = value

    def __get__(self, instance, owner):
        return self.valor


def criar_classe_campo(spec):

    nome = spec.get('nome')
    inicio =  spec.get('posicao_inicio') - 1
    fim = spec.get('posicao_fim')

    attrs = {
        'nome': nome,
        'inicio': inicio,
        'fim': fim,
        'digitos': fim - inicio,
        'formato': spec.get('formato', 'alfa'),
        'decimais': spec.get('decimais', 0),
        'default': spec.get('default'),
        }

    return type(nome.encode('utf8'), (CampoBase,), attrs)


class RegistroBase(object):

    def __new__(cls, **kwargs):
        campos = OrderedDict()
        attrs = {'_campos': campos}
        for Campo in cls._campos_cls.values():
            campo = Campo()
            campos.update({campo.nome: campo})
            attrs.update({campo.nome: campo})
        new_cls = type(cls.__name__, (cls, ), attrs)
        return super(RegistroBase, cls).__new__(new_cls, **kwargs)

    def __init__(self, **kwargs):
        self.fromdict(kwargs)

    def necessario(self):
        for campo in self._campos.values():
            eh_controle = campo.nome.startswith('controle_') or\
                          campo.nome.startswith('servico_')
            if not eh_controle and campo.valor != None:
                return True
        return False

    def todict(self):
        data_dict = dict()
        for campo in self._campos.values():
            if campo.valor is not None:
                data_dict[campo.nome] = campo.valor
        return data_dict

    def fromdict(self, data_dict):
        ignore_fields = lambda key: any((
            key.startswith('vazio'),
            key.startswith('servico_'),
            key.startswith('controle_'),
            ))
        for key, value in data_dict.items():
            if hasattr(self, key) and not ignore_fields(key):
                setattr(self, key, value)

    def carregar(self, registro_str):
        for campo in self._campos.values():
            valor = registro_str[campo.inicio:campo.fim].strip()
            if campo.decimais:
                exponente = campo.decimais * -1
                dec = valor[:exponente] + '.' + valor[exponente:]
                try:
                    campo.valor = Decimal(dec)
                except InvalidOperation:
                    raise # raise custom?
            elif campo.formato == 'num':
                campo.valor = int(valor) if valor else campo.default or 0
            else:
                campo.valor = valor

    def __unicode__(self):
        return ''.join([unicode(campo) for campo in self._campos.values()])

class Registro(object):

    def __init__(self, importacao_dirpath):
        #   posicao_inicio, posicao_fim, formato (alpha), decimais (0),
        #   default (zeros se numerico ou brancos se alfa)
        registro_filepath_list = iglob(os.path.join(importacao_dirpath, '*.json'))
        for registro_filepath in registro_filepath_list:
            registro_file = open(registro_filepath)
            spec = json.load(registro_file)
            registro_file.close()
            setattr(self, spec.get('nome'), self.criar_classe_registro(spec))

    def criar_classe_registro(self, spec):
        campos = OrderedDict()
        attrs = {'_campos_cls': campos}
        cls_name = spec.get('nome').encode('utf8')

        campo_specs = spec.get('campos', {})
        for key in sorted(campo_specs.iterkeys()):
            Campo = criar_classe_campo(campo_specs[key])
            entrada = {Campo.nome: Campo}
            campos.update(entrada)
        return type(cls_name, (RegistroBase, ), attrs)

    def registro_decompoe(self, especificacao, string_file):
        especificacao.carregar(string_file)
        return especificacao

==> sigutils-3.1.11.tar.gz: sigutils-3.1.11/SigUtils/registro.py <==
# -*- encoding: utf-8 -*-
#- coding: utf-8
__Autor__ = "Tiago Prates"
__Empresa__ = "Sig Informatica Ltda"
__Site__ = "http://www.siginformatica.com.br"
__Versao__ = "1.00"
__DataAlteracao__ = "17/07/2013"

import sys
import json
from glob import iglob
from collections import OrderedDict
import os
from decimal import Decimal, InvalidOperation

if __name__ == '__main__':
    try:
        if sys.argv[1] == '-9':
            print(('registro.py\t\t\tVersao:' + __Versao__ + '\tData:'+ __DataAlteracao__) +
                  '\tCompoe e decompoe registros de arquivo texto')
        else:
            print(('\n\nregistro.py - Compoe e decompoe registros de arquivo texto \n' +
                   'Versao: '+ __Versao__ + '\n' + 'Data: '+ __DataAlteracao__ + '\n'))
    except:
        print(('\n\nregistro.py - Compoe e decompoe registros de arquivo texto \n' + 'Versao: ' +
               __Versao__ + '\n' + 'Data: '+ __DataAlteracao__ + '\n'))

class CampoBase(object):
    def __init__(self):
        self._valor = None

    @property
    def valor(self):
        return self._valor

    @valor.setter
    def valor(self, valor):
        self._valor = valor


    def __unicode__(self):
        if self.valor is None:
            if self.default is not None:
                if self.decimais:
                    self.valor = Decimal('{0:0.{1}f}'.format(self.default,
                        self.decimais))
                else:
                    self.valor = self.default
                #            else:
                #                raise errors.CampoObrigatorioError(self.nome)

        if self.formato == 'alfa' or self.decimais:
            if self.decimais:
                valor = str(self.valor).replace('.', '')
                chars_faltantes = self.digitos - len(valor)
                return ('0' * chars_faltantes) + valor
            else:
                chars_faltantes = self.digitos - len(self.valor)
                return self.valor + (' ' * chars_faltantes)

        return '{0:0{1}d}'.format(self.valor, self.digitos)

    def __repr__(self):
        return str(self)

    def __set__(self, instance, value):
        self.valor = value

    def __get__(self, instance, owner):
        return self.valor


def criar_classe_campo(spec):

    nome = spec.get('nome')
    inicio =  spec.get('posicao_inicio') - 1
    fim = spec.get('posicao_fim')

    attrs = {
        'nome': nome,
        'inicio': inicio,
        'fim': fim,
        'digitos': fim - inicio,
        'formato': spec.get('formato', 'alfa'),
        'decimais': spec.get('decimais', 0),
        'default': spec.get('default'),
        }

    return type(nome.encode('utf8'), (CampoBase,), attrs)


class RegistroBase(object):

    def __new__(cls, **kwargs):
        campos = OrderedDict()
        attrs = {'_campos': campos}
        for Campo in list(cls._campos_cls.values()):
            campo = Campo()
            campos.update({campo.nome: campo})
            attrs.update({campo.nome: campo})
        new_cls = type(cls.__name__, (cls, ), attrs)
        return super(RegistroBase, cls).__new__(new_cls, **kwargs)

    def __init__(self, **kwargs):
        self.fromdict(kwargs)

    def necessario(self):
        for campo in list(self._campos.values()):
            eh_controle = campo.nome.startswith('controle_') or\
                          campo.nome.startswith('servico_')
            if not eh_controle and campo.valor != None:
                return True
        return False

    def todict(self):
        data_dict = dict()
        for campo in list(self._campos.values()):
            if campo.valor is not None:
                data_dict[campo.nome] = campo.valor
        return data_dict

    def fromdict(self, data_dict):
        ignore_fields = lambda key: any((
            key.startswith('vazio'),
            key.startswith('servico_'),
            key.startswith('controle_'),
            ))
        for key, value in list(data_dict.items()):
            if hasattr(self, key) and not ignore_fields(key):
                setattr(self, key, value)

    def carregar(self, registro_str):
        for campo in list(self._campos.values()):
            valor = registro_str[campo.inicio:campo.fim].strip()
            if campo.decimais:
                exponente = campo.decimais * -1
                dec = valor[:exponente] + '.' + valor[exponente:]
                try:
                    campo.valor = Decimal(dec)
                except InvalidOperation:
                    raise # raise custom?
            elif campo.formato == 'num':
                campo.valor = int(valor) if valor else campo.default or 0
            else:
                campo.valor = valor

    def __unicode__(self):
        return ''.join([str(campo) for campo in self._campos.values()])

class Registro(object):

    def __init__(self, importacao_dirpath):
        #   posicao_inicio, posicao_fim, formato (alpha), decimais (0),
        #   default (zeros se numerico ou brancos se alfa)
        registro_filepath_list = iglob(os.path.join(importacao_dirpath, '*.json'))
        for registro_filepath in registro_filepath_list:
            registro_file = open(registro_filepath)
            spec = json.load(registro_file)
            registro_file.close()
            setattr(self, spec.get('nome'), self.criar_classe_registro(spec))

    def criar_classe_registro(self, spec):
        campos = OrderedDict()
        attrs = {'_campos_cls': campos}
        cls_name = spec.get('nome').encode('utf8')

        campo_specs = spec.get('campos', {})
        for key in sorted(campo_specs.keys()):
            Campo = criar_classe_campo(campo_specs[key])
            entrada = {Campo.nome: Campo}
            campos.update(entrada)
        return type(cls_name, (RegistroBase, ), attrs)

    def registro_decompoe(self, especificacao, string_file):
        especificacao.carregar(string_file)
        return especificacao
